@@ -143,17 +143,18 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
 /******************************************************************************/
 
 /* These R-structures will be allocated and returned to R*/
-  SEXP yout, yout2, ISTATE, RWORK, IROOT;    
+  SEXP yout, yout2=NULL, ISTATE, RWORK, IROOT=NULL;    
 
-  int  i, j, k, l, m, ij, nt, repcount, latol, lrtol, lrw, liw, isOut, maxit, solver;
-  double *xytmp, *rwork, tin, tout, *Atol, *Rtol, *out, *dy, ss;
+  int  i, j, k, l, m, ll, ij, nt, repcount, latol, lrtol, lrw, liw, isOut, maxit, solver;
+  double *xytmp, *rwork, tin, tout, *Atol, *Rtol, *out, *dy=NULL, ss;
   int neq, itol, itask, istate, iopt, *iwork, jt, mflag, nout, ntot, is;
-  int nroot, *jroot, isroot, *ipar, lrpar, lipar, isDll, type, nspec, nx, ny, Nt;
+  int nroot, *jroot=NULL, isroot, *ipar, lrpar, lipar, isDll;
+  int type, nspec, nx, ny, nz, Nt, bndx, bndy, isp;
   
   deriv_func *derivs;
-  jac_func   *jac;
+  jac_func   *jac=NULL;
   jac_vec    *jacvec;
-  root_func  *root;
+  root_func  *root=NULL;
   init_func  *initializer;
     
 /******************************************************************************/
@@ -251,7 +252,7 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
        k = 1;
        for( i = 0; i<nspec; i++) {
          for( j = 0; j<nx; j++) {       
-          if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw ",liw);                  
+          if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
                        iwork[ij++] = k;
            if (j<nx-1) iwork[ij++] = k+1 ;
            if (j>0)    iwork[ij++] = k-1 ;
@@ -269,22 +270,33 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
      else if (type == 3) { /* 2-D problem */
        nspec = INTEGER(Type)[1]; /* number components*/ 
        nx    = INTEGER(Type)[2]; /* dimension x*/ 
-       ny    = INTEGER(Type)[3]; /* dimension y*/     
+       ny    = INTEGER(Type)[3]; /* dimension y*/
+       bndx  = INTEGER(Type)[4]; /* cyclic boundary x*/
+       bndy  = INTEGER(Type)[5]; /* cyclic boundary y*/
        Nt    = nx*ny;
        ij     = 31+neq;
        iwork[30] = 1;
        m = 1; 
        for( i = 0; i<nspec; i++) {
+         isp = i*Nt;
          for( j = 0; j<nx; j++) {       
            for( k = 0; k<ny; k++) {       
-           if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw ",liw);                  
+           if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
                                 iwork[ij++] = m;
              if (k<ny-1)        iwork[ij++] = m+1;
+
              if (j<nx-1)        iwork[ij++] = m+ny;
              if (j >0)          iwork[ij++] = m-ny;
              if (k >0)          iwork[ij++] = m-1;
-
-            for(l = 0; l<nspec;l++) 
+             if (bndx == 1) {
+               if (j == 0)      iwork[ij++] = isp+(nx-1)*ny+k+1;
+               if (j == nx-1)   iwork[ij++] = isp+k+1;
+             }
+             if (bndy == 1) {
+               if (k == 0)      iwork[ij++] = isp+(j+1)*ny;
+               if (k == ny-1)   iwork[ij++] = isp + j*ny +1;
+             }
+            for(l = 0; l<nspec;l++)
               if (l != i)       iwork[ij++] = l*Nt+j*ny+k+1;
        
             iwork[30+m] = ij-30-neq;
@@ -293,7 +305,41 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
           }
         }   
        }
-    }  
+     else if (type == 4) { /* 3-D problem */
+       nspec = INTEGER(Type)[1]; /* number components*/
+       nx    = INTEGER(Type)[2]; /* dimension x*/
+       ny    = INTEGER(Type)[3]; /* dimension y*/
+       nz    = INTEGER(Type)[4]; /* dimension y*/
+/*       bndx  = INTEGER(Type)[5];
+       bndy  = INTEGER(Type)[6];  cyclic boundary NOT yet implemented*/
+       Nt    = nx*ny*nz;
+       ij     = 31+neq;
+       iwork[30] = 1;
+       m = 1;
+       for( i = 0; i<nspec; i++) {
+         isp = i*Nt;
+         for( j = 0; j<nx; j++) {
+           for( k = 0; k<ny; k++) {
+             for( ll = 0; ll<nz; ll++) {
+              if (ij > liw-4)  error ("not enough memory allocated in iwork - increase liw %i ",liw);
+                                 iwork[ij++] = m;
+              if (ll<nz-1)       iwork[ij++] = m+1;
+              if (k<ny-1)        iwork[ij++] = m+nz;
+              if (j<nx-1)        iwork[ij++] = m+ny*nz;
+
+              if (j >0)          iwork[ij++] = m-ny*nz;
+              if (k >0)          iwork[ij++] = m-nz;
+              if (ll >0)         iwork[ij++] = m-1;
+              for(l = 0; l<nspec;l++)
+                if (l != i)       iwork[ij++] = l*Nt+j*ny*nz+k*nz+ll+1;
+
+              iwork[30+m] = ij-30-neq;
+              m = m+1;
+              }
+            }
+          }
+        }
+       }    }
 
 /* initialise global R-variables... */
 
@@ -433,7 +479,7 @@ SEXP call_lsoda(SEXP y, SEXP times, SEXP func, SEXP parms, SEXP rtol,
     
 	  if (istate == -1) 
      {
-      warning("an excessive amount of work (> maxsteps ) was done, but integration was successful - increase maxsteps");
+      warning("an excessive amount of work (> maxsteps ) was done, but integration was not successful - increase maxsteps");
      }
 	  if (istate == 3 && solver == 4)
 	    { istate = -20;  repcount = 50;  
